#!/usr/bin/env python

"""
Plot the change in value of oil in various mediums of exchange
"""

import pylab
from matplotlib.dates import YearLocator, MonthLocator, DateFormatter
from matplotlib.numerix.mlab import array, cumsum
import datetime



#####################
# Config Options
#####################
oil_data = open("kushing_oil_prices_1986-2008.txt")                
gold_data = open("london_fixed_gold_prices_1986-2008.txt")
euro_data = open("dollar_to_euro_2000-2008.txt")
start_date = datetime.date(2000,1,4)
end_date = datetime.date(2008,1,2)
do_points = False
point_size = 1
line_style = "-"
marker = "."
do_regression_lines = False
do_moving_average = True
average_period = 20 #Make it even
gold_color = "orange"
dollar_color = "green"
euro_color = "red"
resolution = (1024,768)
image_filename = "change_in_oil_price.png"

def movavg(s, n):
    """
    returns an n period moving average for the time series s
       
    s is a list ordered from oldest (index 0) to most recent (index -1)
    n is an integer

        returns a numeric array of the moving average

    See also ema in this module for the exponential moving average.
    """
    s = array(s)
    c = cumsum(s)
    return (c[n-1:] - c[:-n+1]) / float(n-1)

if __name__ == "__main__":
    
    one_AD = datetime.date(1,1,1)
    start_day = (start_date - one_AD).days
    end_day = (end_date - one_AD).days

    #Read oil values
    oil_daily_dollar_value = {}
    for line in oil_data:
        date, value = line.strip().split("|")
        date_parts = tuple([int(x) for x in date.split("-")])
        date = datetime.date(*date_parts)
        if (date < start_date) or (date > end_date):
            #Date out of range, skip it.
            continue
        day = (date - one_AD).days
        try:
            value = float(value)
        except ValueError:
            #Invalid value, skip this day.
            continue
        oil_daily_dollar_value[day] = value

    #Read gold values
    gold_daily_dollar_value = {}
    gold_daily_euro_value = {}
    for line in gold_data:
        date, dollar_value, euro_value = line.strip().split("|")
        date_parts = tuple([int(x) for x in date.split("-")])
        date = datetime.date(*date_parts)
        if (date < start_date) or (date > end_date):
            #Date out of range, skip it.
            continue
        day = (date - one_AD).days
        try:
            dollar_value = float(dollar_value)
            gold_daily_dollar_value[day] = dollar_value
        except ValueError:
            #Invalid dollar value, skip this day.
            continue
        try:
            euro_value = float(euro_value)
            gold_daily_euro_value[day] = euro_value
        except ValueError:
            #Invalid euro value, skip this day.
            continue

    #Read dollar to euro exchange rates
    euro_daily_dollar_value = {}
    for line in euro_data:
        date, value = line.strip().split("|")
        date_parts = tuple([int(x) for x in date.split("-")])
        date = datetime.date(*date_parts)
        if (date < start_date) or (date > end_date):
            #Date out of range, skip it.
            continue
        day = (date - one_AD).days
        try:
            value = float(value)
        except ValueError:
            #Invalid value, skip this day.
            continue
        euro_daily_dollar_value[day] = value

    #Calculate the value of a barrel in ounces of gold
    oil_daily_gold_value = {}
    for day in oil_daily_dollar_value.keys():
        try:
            barrel_dollar_value = oil_daily_dollar_value[day]
            gold_dollar_value = gold_daily_dollar_value[day]
        except KeyError:
            #If either value doesn't exist, skip this day
            continue
        #How much gold can be bought for the barrel value?
        gold_amount = barrel_dollar_value / gold_dollar_value
        oil_daily_gold_value[day] = gold_amount

    #Calculate the daily gold value of a barrel, relative to the start value.
    oil_initial_gold_value = oil_daily_gold_value[start_day]
    oil_daily_gold_value_relative = {}
    for day, value in oil_daily_gold_value.items():
        oil_daily_gold_value_relative[day] = (value / oil_initial_gold_value) * 100

    #Calculate the daily dollar value of a barrel, relative to the start value.
    oil_initial_dollar_value = oil_daily_dollar_value[start_day]
    oil_daily_dollar_value_relative = {}
    for day, value in oil_daily_dollar_value.items():
        oil_daily_dollar_value_relative[day] = (value / oil_initial_dollar_value) * 100

    #Calculate the daily euro value of a barrel, relative to the start value.

    ### We can do euro values against the dollar or against gold
    #Against the dollar
#     oil_initial_euro_value = oil_daily_dollar_value[start_day] * euro_daily_dollar_value[start_day]
#     oil_daily_euro_value_relative = {}
#     for day, dollar_value in oil_daily_dollar_value.items():
#         euro_value = dollar_value * euro_daily_dollar_value[day]
#         oil_daily_euro_value_relative[day] = euro_value / oil_initial_euro_value
    #Against gold
    oil_initial_euro_value = oil_daily_gold_value[start_day] * gold_daily_euro_value[start_day]
    oil_daily_euro_value_relative = {}
    for day, gold_value in oil_daily_gold_value.items():
        euro_value = gold_value * gold_daily_euro_value[day]
        oil_daily_euro_value_relative[day] = (euro_value / oil_initial_euro_value) * 100

    years = YearLocator()
    months = MonthLocator()
    yearsFmt = DateFormatter("'%y")
    
    ax = pylab.subplot(111, axisbg="0.15")

    ###################################
    # Plot Change in dollar value of oil
    ###################################
    dollar_dates = []
    dollar_values = []
    for day, value in oil_daily_dollar_value_relative.items():
        dollar_dates.append(day)
        dollar_values.append(value)
    if do_points:
        pylab.plot_date(dollar_dates, dollar_values,  ms=point_size, linestyle=line_style, color=dollar_color)
    #Linear Regression
    if do_regression_lines:
        (ar,br) = pylab.polyfit(dollar_dates, dollar_values, 1)
        dollar_regression=pylab.polyval([ar,br],dollar_dates)
        pylab.plot(dollar_dates,dollar_regression,'r.-', color=dollar_color)
    #Moving Average
    if do_moving_average:
        ma = movavg(dollar_values,average_period)
        pylab.plot(dollar_dates[(average_period/2)-1:-1*(average_period/2)],
                   ma, 'b', linewidth=1, color=dollar_color,
                   label="US Dollar")

    ###################################
    # Plot Change in euro value of oil
    ###################################
    euro_dates = []
    euro_values = []
    for day, value in oil_daily_euro_value_relative.items():
        euro_dates.append(day)
        euro_values.append(value)
    if do_points:
        pylab.plot_date(euro_dates, euro_values,  ms=point_size, linestyle=line_style, color=euro_color)
    #Linear Regression
    if do_regression_lines:
        (ar,br) = pylab.polyfit(euro_dates, euro_values, 1)
        euro_regression=pylab.polyval([ar,br],euro_dates)
        pylab.plot(euro_dates,euro_regression,'r.-', color=euro_color)
    #Moving Average
    if do_moving_average:
        ma = movavg(euro_values,average_period)
        pylab.plot(euro_dates[(average_period/2)-1:-1*(average_period/2)],
                   ma, 'b', linewidth=1, color=euro_color,
                   label="Euro")

    ###################################
    # Plot Change in gold value of oil
    ###################################
    gold_dates = []
    gold_values = []
    for day, value in oil_daily_gold_value_relative.items():
        gold_dates.append(day)
        gold_values.append(value)
    if do_points:
        pylab.plot_date(gold_dates, gold_values, ms=point_size, linestyle=line_style, color=gold_color)
    #Linear Regression
    if do_regression_lines:
        (ar,br) = pylab.polyfit(gold_dates, gold_values, 1)
        gold_regression=pylab.polyval([ar,br],gold_dates)
        pylab.plot(gold_dates,gold_regression,'r.-', color=gold_color)
    #Moving Average
    if do_moving_average:
        ma = movavg(gold_values,average_period)
        pylab.plot(gold_dates[(average_period/2)-1:-1*(average_period/2)],
                   ma, 'b', linewidth=1, color=gold_color,
                   label="Gold")
    


    #Format the ticks
    ax.xaxis.set_major_locator(years)
    ax.xaxis.set_major_formatter(yearsFmt)
    ax.xaxis.set_minor_locator(months)
    ax.autoscale_view()

    #Format the coords message box
    def price(x): return '$%1.2f'%x
    ax.format_xdata = DateFormatter('%Y-%m-%d')
    ax.format_ydata = price

    pylab.title("Change in the price of crude oil since year 2000. \n(Lower values are better.)",color="white")
    pylab.xlabel("Year", color="white")
    pylab.ylabel("Percent growth since 2000", color="white")
    pylab.legend(loc="upper left")


    pylab.grid(True, linewidth=0.05, color="gray")
    
    width = resolution[0]/150.0
    height = resolution[1]/150.0
    fig = pylab.figure(num=1)
    fig.set_dpi(150.0)
    fig.set_size_inches(width, height)

    pylab.yticks(color="white")
    pylab.xticks(color="white")
    
    
    pylab.savefig(image_filename, facecolor="black")
    print "Saved %(image_filename)s" % locals()
    
